]> git.sesse.net Git - stockfish/blobdiff - src/tt.h
Retire 'os' flag from Makefile
[stockfish] / src / tt.h
index eeb18e32acff0510656537dad8cdac29b4a8f1f6..534409f32800bfc34110d931cac3bd29e0827cfe 100644 (file)
--- a/src/tt.h
+++ b/src/tt.h
 #include "misc.h"
 #include "types.h"
 
-/// The TTEntry is the 14 bytes transposition table entry, defined as below:
+/// The TTEntry is the 10 bytes transposition table entry, defined as below:
 ///
-/// key        32 bit
+/// key        16 bit
 /// move       16 bit
-/// bound type  8 bit
-/// generation  8 bit
 /// value      16 bit
-/// depth      16 bit
 /// eval value 16 bit
+/// generation  6 bit
+/// bound type  2 bit
+/// depth       8 bit
 
 struct TTEntry {
 
   Move  move()  const      { return (Move )move16; }
-  Bound bound() const      { return (Bound)bound8; }
   Value value() const      { return (Value)value16; }
-  Depth depth() const      { return (Depth)depth16; }
   Value eval_value() const { return (Value)evalValue; }
+  Depth depth() const      { return (Depth)depth8; }
+  Bound bound() const      { return (Bound)(genBound8 & 0x3); }
 
 private:
   friend class TranspositionTable;
 
-  void save(uint32_t k, Value v, Bound b, Depth d, Move m, uint8_t g, Value ev) {
+  void save(uint16_t k, Value v, Bound b, Depth d, Move m, uint8_t g, Value ev) {
 
-    key32       = (uint32_t)k;
-    move16      = (uint16_t)m;
-    bound8      = (uint8_t)b;
-    generation8 = (uint8_t)g;
-    value16     = (int16_t)v;
-    depth16     = (int16_t)d;
-    evalValue   = (int16_t)ev;
+    key16     = (uint16_t)k;
+    move16    = (uint16_t)m;
+    value16   = (int16_t)v;
+    evalValue = (int16_t)ev;
+    genBound8 = (uint8_t)(g | b);
+    depth8    = (int8_t)d;
   }
 
-  uint32_t key32;
+  uint16_t key16;
   uint16_t move16;
-  uint8_t bound8, generation8;
-  int16_t value16, depth16, evalValue;
+  int16_t  value16;
+  int16_t  evalValue;
+  uint8_t  genBound8;
+  int8_t   depth8;
 };
 
+/// TTCluster is a 32 bytes cluster of TT entries consisting of:
+///
+/// 3 x TTEntry (3 x 10 bytes)
+/// padding     (2 bytes)
+
+const unsigned TTClusterSize = 3;
+
+struct TTCluster {
+  TTEntry entry[TTClusterSize];
+  char padding[2];
+};
 
 /// A TranspositionTable consists of a power of 2 number of clusters and each
/// cluster consists of ClusterSize number of TTEntry. Each non-empty entry
 /// contains information of exactly one position. The size of a cluster should
 /// not be bigger than a cache line size. In case it is less, it should be padded
 /// to guarantee always aligned accesses.
 
 class TranspositionTable {
 
-  static const unsigned ClusterSize = 4;
-
 public:
  ~TranspositionTable() { free(mem); }
-  void new_search() { ++generation; }
+  void new_search() { generation += 4; } // Lower 2 bits are used by Bound
 
   const TTEntry* probe(const Key key) const;
   TTEntry* first_entry(const Key key) const;
-  void resize(uint64_t mbSize);
+  void resize(size_t mbSize);
   void clear();
   void store(const Key key, Value v, Bound type, Depth d, Move m, Value statV);
 
 private:
-  uint32_t hashMask;
-  TTEntry* table;
+  size_t clusterCount;
+  TTCluster* table;
   void* mem;
-  uint8_t generation; // Size must be not bigger than TTEntry::generation8
+  uint8_t generation; // Size must be not bigger than TTEntry::genBound8
 };
 
 extern TranspositionTable TT;
@@ -94,11 +104,11 @@ extern TranspositionTable TT;
 
 /// TranspositionTable::first_entry() returns a pointer to the first entry of
 /// a cluster given a position. The lowest order bits of the key are used to
-/// get the index of the cluster.
+/// get the index of the cluster inside the table.
 
 inline TTEntry* TranspositionTable::first_entry(const Key key) const {
 
-  return table + ((uint32_t)key & hashMask);
+  return &table[(size_t)key & (clusterCount - 1)].entry[0];
 }
 
 #endif // #ifndef TT_H_INCLUDED
@@
 
 /// A TranspositionTable consists of a power of 2 number of clusters and each
-/// cluster consists of ClusterSize number of TTEntry. Each non-empty entry
+/// cluster consists of TTClusterSize number of TTEntry. Each non-empty entry
 /// contains information of exactly one position. The size of a cluster should
 /// not be bigger than a cache line size. In case it is less, it should be padded